# Learning Reinforcement Learning on your own

## Outlook

The pages below contain teaching material from Olivier Sigaud's reinforcement learning (RL) lectures. They are designed to help you learn RL on your own. Each page contains at least slides, and most often lessons in youtube videos and notebooks corresponding to labs. There might also be additional material to go beyond the basics.

The order of the lessons below matters, if you want to understand a lot about RL and you don't have much prior knowledge, you are encouraged to work on these lessons sequentially. Getting a good grasp on RL from the "central flow of lessons" below may take something like 10 days of work to a motivated beginner.

The labs are based on [the bbrl library](https://github.com/osigaud/bbrl), before studying DQN you should have a look at [the bbrl documentation](https://github.com/osigaud/bbrl/tree/master/docs/overview.md) and [the bbrl introductory notebooks](https://github.com/osigaud/bbrl/tree/master/docs/notebooks).

These pages are subject to perpetual improvement, if you have any question or any suggestion to improve the content, send a message to Olivier.Sigaud@sorbonne-universite.fr.

## Reinforcement Learning: Central flow of lessons

[Overview: the 5 routes to Deep RL](https://github.com/osigaud/bbrl/tree/master/docs/learning_RL/overview.md)

------------

### Tabular reinforcement learning

[Tabular dynamic programming](https://github.com/osigaud/bbrl/tree/master/docs/learning_RL/Tabular/tabular_dp.md)

[Tabular model-free reinforcement learning](https://github.com/osigaud/bbrl/tree/master/docs/learning_RL/Tabular/tabular_mfrl.md)

[Tabular model-based reinforcement learning](https://github.com/osigaud/bbrl/tree/master/docs/learning_RL/Tabular/tabular_mbrl.md)

------------

### Deep (model-free) reinforcement learning

[Deep Q-Network (DQN) and its successors](https://github.com/osigaud/bbrl/tree/master/docs/learning_RL/DeepMFRL/dqn.md)

[Deep Deterministic Policy Gradient (DDPG) and Twin Delayed Deep Deterministic Policy Gradient (TD3)](https://github.com/osigaud/bbrl/tree/master/docs/learning_RL/DeepMFRL/ddpg.md)

[On-policy vs Off-policy](https://github.com/osigaud/bbrl/tree/master/docs/learning_RL/DeepMFRL/onp_ofp.md)

[Policy Gradient approaches](https://github.com/osigaud/bbrl/tree/master/docs/learning_RL/DeepMFRL/reinforce.md)

[Bias vs Variance](https://github.com/osigaud/bbrl/tree/master/docs/learning_RL/DeepMFRL/bias_variance.md)

[Advantage Actor-Critic (A2C)](https://github.com/osigaud/bbrl/tree/master/docs/learning_RL/DeepMFRL/a2c.md)

[Trust Region Policy Optimization (TRPO) and ACKTR](https://github.com/osigaud/bbrl/tree/master/docs/DeepMFRL/learning_RL/trpo.md)

[Proximal Policy Optimization (PPO)](https://github.com/osigaud/bbrl/tree/master/docs/learning_RL/DeepMFRL/ppo.md)

[Soft Actor-Critic (SAC)](https://github.com/osigaud/bbrl/tree/master/docs/learning_RL/DeepMFRL/sac.md)

[High UTD ratio algorithms (TQC, DroQ)](https://github.com/osigaud/bbrl/tree/master/docs/learning_RL/DeepMFRL/high_UTD.md)

[Deel Model-free RL Wrap-Up](https://github.com/osigaud/bbrl/tree/master/docs/learning_RL/DeepMFRL/wrap_up.md)

------------

### Direct policy search and RL

[Direct policy search and RL: introduction](https://github.com/osigaud/bbrl/tree/master/docs/learning_RL/EvoRL/intro.md)

[Direct policy search methods](https://github.com/osigaud/bbrl/tree/master/docs/learning_RL/EvoRL/dps_methods.md)

[Policy gradient details](https://github.com/osigaud/bbrl/tree/master/docs/learning_RL/EvoRL/pg_details.md)

[Direct policy search and RL: comparisons](https://github.com/osigaud/bbrl/tree/master/docs/learning_RL/EvoRL/comparisons.md)

[Direct policy search and RL: combinations](https://github.com/osigaud/bbrl/tree/master/docs/learning_RL/EvoRL/combinations.md)

[Population-based training](https://github.com/osigaud/bbrl/tree/master/docs/learning_RL/EvoRL/pbt.md)

[TD-MPC](https://github.com/osigaud/bbrl/tree/master/docs/learning_RL/EvoRL/td_mpc.md)

------------

### Goal-conditioned reinforcement learning

[GCRL: introduction](https://github.com/osigaud/bbrl/tree/master/docs/learning_RL/GCRL/intro.md)

[GCRL: core concepts](https://github.com/osigaud/bbrl/tree/master/docs/learning_RL/GCRL/core_concepts.md)

[GCRL: typology](https://github.com/osigaud/bbrl/tree/master/docs/learning_RL/GCRL/typology.md)

[GCRL: skill learners](https://github.com/osigaud/bbrl/tree/master/docs/learning_RL/GCRL/skill_learners.md)

[GCRL: hindsight experience replay](https://github.com/osigaud/bbrl/tree/master/docs/learning_RL/her.md)

[GCRL: goal reachers](https://github.com/osigaud/bbrl/tree/master/docs/learning_RL/GCRL/goal_reachers.md)

------------

<!---
## Beyond standard RL approaches

Imitation learning

Direct Policy Search (DPS) approaches

Combining DPS and RL
--->
